@@ -1,7 +1,7 @@
 <?php
 function _addComment(&$tof,$vars) {
 	if ($vars['Comment'] != "") {
-		global $REMOTE_ADDR;
+		global $_SERVER['REMOTE_ADDR'];
 		$comment = array('CommentID' => time(),
 						 'PictureID' => $vars['PictureID'],
 						 'Comment' => $vars['Comment'],
