@@ -1,13 +1,14 @@
 <?php
 function _addTopicPerms(&$tof,$vars) {
 	if (is_array($vars['Groups']) && is_array($vars['_Perms']))
-		$this->addTopicPerms(
+		addTopicPerms(
+			$tof,
 			$vars['_TopicID'],
 			$vars['Groups'],
 			$vars['_Perms']);
 }
 
-function addTopicPerms($topicID,$groups,$perms) {
+function addTopicPerms(&$tof,$topicID,$groups,$perms) {
 	foreach($perms as $perm) {
 		  foreach($groups as $groupID) {
 			  $tof->ct->addTopicPerm($topicID,$groupID,$perm);
