@@ -2,7 +2,7 @@
 function updateDisplayRatios(&$tof,$overwrite=false) {
 	$pictures = $tof->ct->getPictures('PictureID,DisplayRatio,RollID,FrameID');
 	foreach($pictures as $picture) {
-		$tof->updateDisplayRatio($picture,$overwrite);
+		updateDisplayRatio($tof,$picture,$overwrite);
 	}
 }
 
