@@ -1,6 +1,8 @@
 <?php
 function _rebuildThumb(&$tof,$vars) {
-	$tof->createThumb(
+	require("actions/thumb.php");
+	createThumb(
+		$tof,
 		$tof->ct->getPicture($vars['PictureID']),
 		true);
 }
