@@ -1,7 +1,7 @@
 <?php
 function maybeCreateRoll(&$tof,$newRollID, $rollID) {
 	if ($newRollID) {
-		$rollID = $this->user['Username'].'/'.$newRollID;
+		$rollID = $tof->user['Username'].'/'.$newRollID;
 		$tof->ct->addRoll(
 			  array('RollID' => $rollID,
 					'Owner' => $tof->uid));
