@@ -1,6 +1,8 @@
 <?php
 function _sendMail(&$tof,$vars) {
-	$tof->sendMail(
+	require_once("actions/mail.php");
+	sendMail(
+		$tof,
 		$vars['Groups'],
 		$vars['MailSubject'],
 		$vars['MailBody'],
