@@ -1,7 +1,8 @@
 <?php
 function _updateGroup(&$tof,$vars) {
+	require_once("actions/groups.php");
 	$group = array( 'GroupID' => $vars['GroupID'],
 					'Name' => $vars['groupName'] );
-	$tof->updateGroup($group,$vars['groupUsers']);
+	updateGroup($tof,$group,$vars['groupUsers']);
 }
 ?>
\ No newline at end of file
