@@ -3,9 +3,14 @@ function _uploadArchive(&$tof,$vars) {
 	require_once("actions/roll.php");
 	require_once("actions/topic.php");
 	require_once("actions/uploads.php");
-	$tof->log("uploadArchive ".$_FILES['file']['name']);
+	$tof->log(sprintf(_('Uploading archive %s'),$_FILES['file']['name']));
 	$rollID = maybeCreateRoll($tof,$vars['NewRollID'],$vars['RollID']);
-	$topicID = maybeCreateTopic($tof,$vars['ParentTopicID'],$vars['NewTopicDescription']);
+	$topicID = maybeCreateTopic(
+		$tof,
+		$vars['ParentTopicID'],
+		$vars['NewTopicDescription'],
+		array(EVERYBODY),
+		array(READ,WRITE));
 	uploadArchive(
 		$tof,
 		$_FILES['file']['tmp_name'],
@@ -15,6 +20,5 @@ function _uploadArchive(&$tof,$vars) {
 		$rollID,
 		$topicID,
 		$vars['Groups']);
-	/* delete Comments */
 }
 ?>
\ No newline at end of file
