@@ -5,15 +5,16 @@ page_open(array("sess" => "Tof_Session",
 
 $tof = new MySlooze($auth->auth['uid'],false);
 require_once("actions/file_data.php");
-switch ($Size) {
+$pictureID = $_REQUEST['PictureID'];
+switch ($_REQUEST['Size']) {
 	case 'thumb': 
-		getFileData($tof,$PictureID,$tof->thumb_suffix);
+		getFileData($tof,$pictureID,$tof->thumb_suffix);
 		break;
 	case 'full':
-		getFileData($tof,$PictureID,$tof->orig_suffix);
+		getFileData($tof,$pictureID,$tof->orig_suffix);
 		break;
 	case 'medium': 
-		getFileData($tof,$PictureID,$tof->picture_suffix);
+		getFileData($tof,$pictureID,$tof->picture_suffix);
 		break;
 }
 
