@@ -4,9 +4,6 @@ function showIconRPC(&$render,$img,$text,$action,$params) {
 }
 
 function showIcon(&$render,$img,$text,$action,$params,$rpc=false) {
-	foreach($array as $name => $value) {
-		$params[$name] = rawurlencode($value);
-	}
 	$params['action'] = $action;
 	$url = $render->actionAndViewURL($params);
 	if ($rpc) {
