@@ -12,7 +12,9 @@ function renderRateSort(&$render,$start=0) {
 	$render->setTitle(_("Rating sort"));
 	$render->topTitle .= " ".countString($render,$start,$count);
 	$render->showSort = false;
+	$render->show('<div class="normal">');
 	showThumbs($render,$pictures,$start,$count);
+	$render->show('</div>');
 }
 
 function renderPictureInRateSort(&$render,$pictureID, $position) {
