@@ -132,6 +132,7 @@ function showThumb(
 {
 	require_once("render/tooltip.php");
 	require_once("render/format_date.php");
+	require_once("render/actions.php");
 
 	if (is_array($picture) && count($picture) > 0) {
 		$fileName = $render->slooze->getThumbFilename($picture);
@@ -143,12 +144,12 @@ function showThumb(
 				$render->show('<div class="move">',false);
 				//$render->show('|< << < > >> >|');
 				if ($position>0)
-					$render->showLink(
+					showLink(
 						$render,
 						'<','moveUp',_('move up'),
 						array('_PictureID'=>$picture['PictureID'],'_TopicID'=>$moveInTopic));
 				if ($position<($render->count-1))
-					$render->showLink(
+					showLink(
 						$render,
 						'>','moveDown',_('move down'),
 						array('_PictureID'=>$picture['PictureID'],'_TopicID'=>$moveInTopic));
