@@ -134,6 +134,7 @@ class SloozeCtSql {
 								  'DisableTooltips'=>'boolean',
 								  'Admin'=>'boolean',
 								  'Selection'=>'boolean',
+								  'Upload' => 'boolean',
 								  'NotifyComment'=>'boolean'),
 				 'topics' => array('ParentTopicID'=>'integer',
 								   'Book' => 'boolean'),
@@ -146,12 +147,28 @@ class SloozeCtSql {
 	  }
   }
 
+  function setRender(&$render) {
+	  $this->render = &$render;
+  }
+
+  function log($str) {
+	  if ($this->render)
+		  $this->render->log($str);
+  } 
+
+  var $inTransaction = 0;
   function begin() {
-	  $this->db->query("BEGIN");
+	  if ($this->inTransaction === 0) {
+		  $this->db->query("BEGIN");
+	  }
+	  $this->inTransaction += 1;
   }
 
   function commit() {
-	  $this->db->query("COMMIT");
+	  $this->inTransaction -= 1;
+	  if ($this->inTransaction === 0) {
+		  $this->db->query("COMMIT");
+	  }
   }
 
   function newDB() {
@@ -293,17 +310,32 @@ class SloozeCtSql {
    * Returns all the topics of a picture. 
    * @param pictureID the picture
    */
-  function getTopics($pictureID,$userID) {
+  function getTopicsOfPicture($pictureID,$userID) {
 	  $this->db->query(
-		  "select * from Topics join topicscontent using (topicid) ".
-		  "where PictureID=$pictureID and Direct=true and ".
+		  "SELECT * FROM Topics JOIN topicscontent USING (topicid)".
+		  " WHERE PictureID=$pictureID AND Direct=true and ".
 		  $this->buildTopicPermsCondition($userID).
-		  " order by Description");
+		  " ORDER BY Description");
 	  return $this->getAllRecords('topics');
   }
 
+  function getTopics($userID,$perms) {
+	  $this->db->query(
+		  "SELECT * FROM Topics".
+		  " WHERE ".$this->buildTopicPermsCondition($userID,$perms).
+		  " ORDER BY Description");
+	  return $this->getAllRecords('topics');
+  }
+
+  function getTopicIDs($userID,$perms) {
+	  $this->db->query(
+		  "SELECT TopicID FROM Topics".
+		  " WHERE ".$this->buildTopicPermsCondition($userID,$perms));
+	  return $this->getAllSingleValues(true);
+  }
+
   /* public: getTopicIDs() returns all the topic IDs of a picture. */
-  function getTopicIDs($pictureID) {
+  function getTopicIDsOfPicture($pictureID) {
 	  $this->db->query(
 		  "select TopicID from Topics join topicscontent using (topicid) ".
 		  "where PictureID=$pictureID and Direct=true ".
@@ -378,7 +410,7 @@ class SloozeCtSql {
 	  $topicID = $this->nextVal('topics_topicid_seq');
 	  $topic['TopicID'] = $topicID;
 	  $cols = $this->makeCols($topic);
-	  $vals = $this->makeVals($topic);
+	  $vals = $this->makeVals($topic,'topics');
 	  $this->db->query("INSERT INTO Topics ($cols) VALUES ($vals)");
 	  return $topicID;
   }
@@ -392,7 +424,8 @@ class SloozeCtSql {
 		  $this->db->query("DELETE FROM TopicsContent WHERE TopicID = '$topicID'");
 		  return true;
 	  } else {
-		  $this->errString = _('Cannot delete: there are pictures in this topic.');
+		  $this->errString = 
+			   sprintf(_('Cannot delete topic %s: it contains pictures.'),$topicID);
 		  return false; /* failed */
 	  }
   }
@@ -414,7 +447,7 @@ class SloozeCtSql {
 		  $pictures = $this->getAllPicturesInTopic($topicID,true,'PictureID');
 		  reset($pictures);
 		  while (list($key,$picture) = each($pictures)) {
-			  $topics = $this->getTopicIDs($picture['PictureID']);
+			  $topics = $this->getTopicIDsOfPicture($picture['PictureID']);
 			  $this->updateTopics($picture['PictureID'],$topics);
 		  }
 	  }
@@ -442,12 +475,13 @@ class SloozeCtSql {
   }
 
   /* public: deleteRoll() deletes one single roll */
-  function deleteRoll($rollID) {
-	  $this->db->query("DELETE FROM Rolls WHERE (RollID = '$rollID')");
-	  if ($this->db->Errno==0) {
+  function deleteRoll($rollID,$force) {
+	  if ($forcce || count($this->getAllPicturesInRoll($rollID,'pictures.PictureID'))==0) {
+		  $this->db->query("DELETE FROM Rolls WHERE (RollID = '$rollID')");
 		  return true;
 	  } else {
-		  $this->errString = _('Cannot delete: there are pictures in this roll.');
+		  $this->errString = 
+			   sprintf(_('Cannot delete roll %s: it contains pictures.'),$rollID);
 		  return false; /* failed */
 	  }
   }
@@ -480,28 +514,19 @@ class SloozeCtSql {
   function getPicturesInRoll($rollID, $userID, $columns='pictures.*', $offset=0, $limit=0) {
 	  $query = $this->picturesQuery($userID,$columns,false);
 	  $query->addFilter("pictures.RollID='$rollID'");
-	  /*
-	  if (func_num_args()>3)
-		  $count = $this->countPicturesInQuery($query);
-	  */
 	  $query->addOrder('pictures.FrameID');
-	  /*
-	  if (func_num_args()>3)
-		  $query->setLimit($offset,$limit);
-	  */
 	  $this->query($query);
-	  /*
-	  if (func_num_args()>3)
-		  return array($count,$this->getAllRecords());
-	  else
-		  return array($this->db->num_rows(),$this->getAllRecords());
-	  */
 	  if (func_num_args()>3)
 		  return array($this->db->num_rows(),$this->getRecords($offset,$limit));
 	  else
 		  return array($this->db->num_rows(),$this->getAllRecords('pictures'));
   }
 
+  function getAllPicturesInRoll($rollID, $columns='pictures.*') {
+	  $this->db->query("SELECT $columns FROM pictures WHERE pictures.RollID='$rollID'");
+	  return $this->getAllRecords('pictures');
+  }
+
   function queryPicturesInTopic($userID,$topicID,$recursive,$sort,$columns='pictures.*') {
 	  $query = $this->picturesQuery($userID,$columns,$sort);
 	  if (!($topicID==0 && $recursive)) {
@@ -1293,10 +1318,19 @@ class SloozeCtSql {
    * Makes a comma and single-quote delimited list
    * of the values of $arr, with special characters escaped 
    */
-  function makeVals($arr) {
+  function makeVals($arr,$table=null) {
+	  $types = $this->tableDefs[$table];
 	  $values = array();
 	  reset($arr);
-	  while (list( ,$value) = each($arr)) {
+	  while (list($key ,$value) = each($arr)) {
+		  if ($types) {
+			  if ($types[$key]=='boolean') {
+				  if ($value===true || $value==1 || $value=='t' || $value=='true')
+					  $value = 'true';
+				  else
+					  $value = 'false';
+			  }
+		  }
 		  $values[] = "'" . addslashes($value) . "'";
 	  }
 	  return (join(",", $values));
@@ -1536,24 +1570,15 @@ class SloozeCtSql {
    * Takes into account the creation of the SelfGroup
    */
   function addUser($user) {
-	  if ($user['Upload']!='true') {
-		  $user['Upload']='false';
-	  }
-	  if ($user['Admin']!='true') {
-		  $user['Admin']='false';
-	  }
-	  if ($user['NotifyComment']!='true') {
-		  $user['NotifyComment']='false';
-	  }
-	  
+	  $this->begin();
 	  /* insert new user into table */
 	  $user['UserID'] = $this->getSequenceNextValue("users_userid_seq");
 	  $user['SelfGroup'] = $this->getSequenceNextValue("groups_groupid_seq");
 	  $cols = $this->makeCols($user);
-	  $vals = $this->makeVals($user);
+	  $vals = $this->makeVals($user,'users');
 	  $this->db->query("insert into Users ($cols) values ($vals)");
 	  
-    /* create the SelfGroup for the user */
+	  /* create the SelfGroup for the user */
 	  $this->addGroup(array('GroupID' => $user['SelfGroup'],
 							'Name' => $user['Name'],
 							'Owner' => 1));
@@ -1563,10 +1588,12 @@ class SloozeCtSql {
 	  
 	  /* insert the new user in the "everybody" group */
 	  $this->addUserInGroup($user['UserID'],1);
+	  $this->commit();
 	  return true;
   }
 
   function deleteUser($userID,$removePictures) {
+	  $this->begin();
 	  if ($removePictures) {
 		  $this->db->query("delete from pictures where owner = $userID");
 		  $this->log(sprintf(_('Removed %d pictures of user %s'),$this->db->affected_rows,$userID));
@@ -1576,6 +1603,7 @@ class SloozeCtSql {
 	  }
 	  $this->db->query("delete from groups where groupid = (select selfgroup from users where UserID=$userID)");
 	  $this->db->query("delete from users where UserID=$userID");
+	  $this->commit();
   }
 
   /**
@@ -1810,13 +1838,20 @@ class SloozeCtSql {
 	  return $this->db->f("nextval");
   }
 
-  /* this is just a hack to fix postgresql broken case sensitiveness */
+  /**
+   * This is just a hack to fix postgresql broken case
+   * sensitiveness. It also removes unused keys in record to avoid
+   * memory waste.
+   */
   function fixColumnNames(&$record) {
 	  if (is_array($record)>0) {
 		  reset($record);
 		  while (list($key,$value) = each($record)) {
 			  if ($this->dico[$key]!="") {
 				  $record[$this->dico[$key]] = $value;
+				  unset($record[$key]);
+			  } else if (is_integer($key)) {
+				  unset($record[$key]);
 			  }
 		  }
 	  }
@@ -1917,12 +1952,6 @@ class SloozeCtSql {
 	  $this->db->query("DELETE FROM topicsfrontpage where TopicID=$topicID and PictureID=$pictureID");
   }
 
-  function log($str) {
-	  global $myRender;
-	  if ($myRender)
-		  $myRender->log($str);
-  } 
-
   function moveUp($pictureID, $topicID, $offset = 1) {
 	  $this->begin();
 	  $this->db->query(
@@ -2015,6 +2044,19 @@ class SloozeCtSql {
 		  " Values ($topicID,(select count(PageNum) FROM pages WHERE topicid=$topicID))");
   }
 
+  function removePage($topicID,$pageNum) {
+	  $this->begin();
+	  $this->db->query(
+		  "DELETE FROM pages where TopicID=$topicID AND PageNum=$pageNum");
+	  $this->db->query(
+		  "UPDATE pages SET PageNum = PageNum-1".
+		  " WHERE TopicID = $topicID AND PageNum > $pageNum");
+	  $this->db->query(
+		  "UPDATE pageelements SET PageNum = PageNum-1".
+		  " WHERE TopicID = $topicID AND PageNum > $pageNum");
+	  $this->commit();
+  }
+
   function getTextElement($topicID,$num,$id) {
 	  $this->db->query(
 		  "SELECT * FROM textelements".
@@ -2039,6 +2081,11 @@ class SloozeCtSql {
 	  return $image['Id'];
   }
 
+  function removePageElement($topicID,$pageNum,$id) {
+	  $this->db->query(
+		  "DELETE FROM pageelements WHERE TopicID=$topicID AND PageNum=$pageNum AND Id=$id");
+  }
+
   function addTextElement($text) {
 	  $this->db->query(
 		  "SELECT COUNT(Id) FROM textelements".
@@ -2077,7 +2124,7 @@ class SloozeCtSql {
 	  $this->db->query("INSERT INTO $table ($columns) VALUES ($values)");
   }
 
-  function upgrade($major,$minor) {
+  function upgrade($newVersion) {
 	  $this->db->query("SELECT * FROM pg_class where relkind = 'r' AND relname = 'params'");
 	  if (!$this->db->next_record()) {
 		  $this->db->query("CREATE TABLE params (Name varchar PRIMARY KEY, Value varchar)");
@@ -2087,7 +2134,7 @@ class SloozeCtSql {
 	  $current = $this->getParameter('dbversion');
 	  if ($current < 1) {
 		  // add topicperms
-		  $this->db->query("START TRANSACTION");
+		  $this->begin();
 		  $this->db->query(
 			  "CREATE TABLE topicperms (".
 			  "TopicID integer NOT NULL REFERENCES topics ON DELETE CASCADE,".
@@ -2105,11 +2152,11 @@ class SloozeCtSql {
 			  $this->db->query("INSERT INTO topicperms VALUES (${topic['TopicID']},'-1','".OWNER."')");
 		  }
 		  $this->setParameter('dbversion',1);
-		  $this->db->query("COMMIT");
+		  $this->commit();
 	  }
 	  if ($current < 2) {
 		  // Add topicscontent.index
-		  $this->db->query("START TRANSACTION");
+		  $this->begin();
 		  $this->db->query("ALTER TABLE topicscontent ADD Index integer");
 		  $this->db->query("SELECT * FROM topicscontent WHERE Direct = true ORDER BY TopicID");
 		  $db = $this->newDB();
@@ -2127,12 +2174,12 @@ class SloozeCtSql {
 				  " TopicID = $topicID AND PictureID = $pictureID AND Direct = true");
 		  }
 		  $this->setParameter('dbversion',2);
-		  $this->db->query("COMMIT");
+		  $this->commit();
 	  }
 
 	  if ($current < 3) {
 		  // Add topicscontent.index
-		  $this->db->query("START TRANSACTION");
+		  $this->begin();
 		  $this->db->query("ALTER TABLE topics ADD Book boolean");
 		  $this->db->query("UPDATE topics SET Book = false");
 		  $this->db->query("ALTER TABLE topics ALTER Book set NOT NULL");
@@ -2145,11 +2192,12 @@ class SloozeCtSql {
 		  $this->db->query("CREATE INDEX session_changed ON active_sessions (changed)");
 
 		  $this->setParameter('dbversion',3);
-		  $this->db->query("COMMIT");
+		  $this->commit();
 	  }
 
 	  if ($current < 4) {
-		  $this->db->query("START TRANSACTION");
+		  $this->begin();
+		  // Added pages for book albums
 		  $this->db->query(
 			  "CREATE TABLE pages (".
 			  "TopicID integer NOT NULL REFERENCES topics,".
@@ -2162,17 +2210,74 @@ class SloozeCtSql {
 			  " SELECT pages.*,topics.width,topics.height".
 			  " FROM pages JOIN topics USING(TopicID)");
 		  $this->setParameter('dbversion',4);
-		  $this->db->query("COMMIT");
+		  $this->commit();
 	  }
 	  if ($current < 5) {
-		  $this->db->query("START TRANSACTION");
+		  $this->begin();
+		  // Drop obsolete columns
 		  $this->db->query("ALTER TABLE users DROP COLUMN picturesize");
 		  $this->db->query("ALTER TABLE users DROP COLUMN pictureinterval");
 		  $this->db->query("ALTER TABLE users DROP COLUMN showemptytopics");
 		  $this->setParameter('dbversion',5);
-		  $this->db->query("COMMIT");
+		  $this->commit();
 	  }
-	  $this->setParameter('version',$major.'.'.$minor);
+	  if ($current < 6) {
+		  $this->begin();
+		  // If a roll is deleted, delete its pictures
+		  $this->db->query(
+			  "ALTER TABLE pictures ADD CONSTRAINT fkey_rollid".
+			  " FOREIGN KEY (RollID) REFERENCES rolls ON DELETE CASCADE;");
+		  $this->setParameter('dbversion',6);
+		  $this->commit();
+	  }
+	  if ($current < 7) {
+		  $this->begin();
+		  // Give write access to everybody on all albums to match
+		  // previous behaviour
+		  $this->db->query(
+			  "SELECT TopicID FROM Topics".
+			  " WHERE NOT (".$this->buildTopicPermsCondition(1,array(WRITE)).")");
+		  $topics = $this->getAllSingleValues();
+		  foreach($topics as $topicID) {
+			  // Give read access to everybody on all topics
+			  $this->db->query("INSERT INTO topicperms VALUES (${topicID},'1','".WRITE."')");
+		  }
+		  $this->setParameter('dbversion',7);
+		  $this->commit();
+	  }
+	  if ($current < 8) {
+		  $this->begin();
+		  $this->db->query(
+			  "CREATE TABLE pageelements ("
+			  . "	TopicID integer,"
+			  . "	PageNum smallint,"
+			  . "	Id smallint,"
+			  . "	Box box NOT NULL,"
+			  . "	PRIMARY KEY (TopicID,PageNum,Id),"
+			  . "	FOREIGN KEY (TopicID,PageNum) REFERENCES pages"
+			  . "	ON DELETE cascade DEFERRABLE INITIALLY DEFERRED)");
+
+		  $this->db->query(
+			  "CREATE TABLE imageelements ("
+			  . "	PictureID integer REFERENCES pictures,"
+			  . "	HAlign float DEFAULT 0.5,"
+			  . "	VAlign float DEFAULT 0.5"
+			  . ") INHERITS (pageelements)");
+
+		  $this->db->query(
+			  "CREATE TABLE textelements ("
+			  . "	Text character varying,"
+			  . "	Size smallint,"
+			  . "	Font character varying,"
+			  . "	Align character"
+			  . ") INHERITS (pageelements)");
+		  $this->setParameter('dbversion',8);
+		  $this->commit();
+	  }
+
+	  $this->log(sprintf(_('Database upgrade to version %s.'),$this->getParameter('dbversion')));
+
+	  $this->setParameter('version',$newVersion);
   }
 
   function getParameter($name) {
